(function () {

/* Imports */
var Meteor = Package.meteor.Meteor;
var _ = Package.underscore._;
var Random = Package.random.Random;
var SandstormDb = Package['sandstorm-db'].SandstormDb;
var MongoInternals = Package.mongo.MongoInternals;
var Mongo = Package.mongo.Mongo;
var Accounts = Package['accounts-base'].Accounts;
var AccountsServer = Package['accounts-base'].AccountsServer;
var check = Package.check.check;
var Match = Package.check.Match;

(function(){

////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                    //
// packages/accounts-identity/accounts-identity-server.js                                             //
//                                                                                                    //
////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                      //
// Sandstorm - Personal Cloud Sandbox                                                                 // 1
// Copyright (c) 2015 Sandstorm Development Group, Inc. and contributors                              // 2
// All rights reserved.                                                                               // 3
//                                                                                                    // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                    // 5
// you may not use this file except in compliance with the License.                                   // 6
// You may obtain a copy of the License at                                                            // 7
//                                                                                                    // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                       // 9
//                                                                                                    // 10
// Unless required by applicable law or agreed to in writing, software                                // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                  // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                           // 13
// See the License for the specific language governing permissions and                                // 14
// limitations under the License.                                                                     // 15
                                                                                                      // 16
function linkIdentityToAccountInternal(identityId, accountId) {                                       // 17
  // Links the identity to the account. If the account is a demo account, makes the account durable   // 18
  // and gives the identity login access to it.                                                       // 19
                                                                                                      // 20
  check(identityId, String);                                                                          // 21
  check(accountId, String);                                                                           // 22
                                                                                                      // 23
  var accountUser = Meteor.users.findOne({_id: accountId});                                           // 24
  if (!accountUser) {                                                                                 // 25
    throw new Meteor.Error(404, "No account found with ID " + accountId);                             // 26
  }                                                                                                   // 27
                                                                                                      // 28
  if (accountUser.profile) {                                                                          // 29
    throw new Meteor.Error(400, "Cannot link an identity to another identity.");                      // 30
  }                                                                                                   // 31
                                                                                                      // 32
  var identityUser = Meteor.users.findOne({_id: identityId});                                         // 33
                                                                                                      // 34
  if (!identityUser) {                                                                                // 35
    throw new Meteor.Error(404, "No identity found with ID " + identityId);                           // 36
  }                                                                                                   // 37
                                                                                                      // 38
  if (!identityUser.profile) {                                                                        // 39
    throw new Meteor.Error(400, "Cannot link an account to another account");                         // 40
  }                                                                                                   // 41
                                                                                                      // 42
  if (Meteor.users.findOne({"loginIdentities.id": identityUser._id})) {                               // 43
    throw new Meteor.Error(403,                                                                       // 44
                           "Cannot link an identity that can already log into another account");      // 45
  }                                                                                                   // 46
                                                                                                      // 47
  var modifier;                                                                                       // 48
  if (accountUser.expires) {                                                                          // 49
    if (Meteor.users.findOne({"nonloginIdentities.id": identityUser._id})) {                          // 50
      throw new Meteor.Error(403, "Cannot create an account for an identity that's " +                // 51
                                  "already linked to another account.");                              // 52
    }                                                                                                 // 53
                                                                                                      // 54
    modifier = {$push: {loginIdentities: {id: identityUser._id}},                                     // 55
                $unset: {expires: 1},                                                                 // 56
                $set: {upgradedFromDemo: Date.now()}};                                                // 57
  } else {                                                                                            // 58
    modifier = {$push: {nonloginIdentities: {id: identityUser._id}}};                                 // 59
  }                                                                                                   // 60
                                                                                                      // 61
  // Make sure not to add the same identity twice.                                                    // 62
  Meteor.users.update({_id: accountUser._id, "nonloginIdentities.id": {$ne: identityUser._id},        // 63
                       "loginIdentities.id": {$ne: identityUser._id}},                                // 64
                      modifier);                                                                      // 65
                                                                                                      // 66
  if (accountUser.expires) {                                                                          // 67
    var demoIdentityId = SandstormDb.getUserIdentityIds(accountUser)[0];                              // 68
    Meteor.users.update({_id: demoIdentityId},                                                        // 69
                        {$unset: {expires: 1},                                                        // 70
                         $set: {upgradedFromDemo: Date.now()}});                                      // 71
                                                                                                      // 72
    // Mark the demo identity as nonlogin. It'd be nicer if the identity started out as nonlogin,     // 73
    // but to get that to work we would need to adjust the account creation and first login logic.    // 74
    Meteor.users.update({_id: accountUser._id,                                                        // 75
                         "loginIdentities.id": demoIdentityId,                                        // 76
                         "nonloginIdentities.id": {$not: {$eq: demoIdentityId}}},                     // 77
                        {$pull: {loginIdentities: {id: demoIdentityId}},                              // 78
                         $push: {nonloginIdentities: {id: demoIdentityId}}});                         // 79
                                                                                                      // 80
  }                                                                                                   // 81
}                                                                                                     // 82
                                                                                                      // 83
Meteor.methods({                                                                                      // 84
  loginWithIdentity: function (accountUserId) {                                                       // 85
    // Logs into the account with ID `accountUserId`. Throws an exception if the current user is      // 86
    // not an identity user listed in the account's `loginIdentities` field. This method is not       // 87
    // intended to be called directly; client-side code should only invoke it through                 // 88
    // `Meteor.loginWithIdentity()`, which additionally maintains the standard Meteor client-side     // 89
    // login state.                                                                                   // 90
                                                                                                      // 91
    check(accountUserId, String);                                                                     // 92
                                                                                                      // 93
    var identityUser = Meteor.user();                                                                 // 94
    if (!identityUser || !identityUser.profile) {                                                     // 95
      throw new Meteor.Error(403, "Must be already logged in as an identity.");                       // 96
    }                                                                                                 // 97
                                                                                                      // 98
    var accountUser = Meteor.users.findOne(accountUserId);                                            // 99
    if (!accountUser) {                                                                               // 100
      throw new Meteor.Error(404, "No such user found: " + accountUserId);                            // 101
    }                                                                                                 // 102
                                                                                                      // 103
    var linkedIdentity = _.findWhere(accountUser.loginIdentities, {id: identityUser._id});            // 104
                                                                                                      // 105
    if (!linkedIdentity) {                                                                            // 106
      throw new Meteor.Error(403, "Current identity is not a login identity for account "             // 107
                             + accountUserId);                                                        // 108
    }                                                                                                 // 109
                                                                                                      // 110
    return Accounts._loginMethod(this, "loginWithIdentity", [accountUserId],                          // 111
                                 "identity", function () { return { userId: accountUserId }; });      // 112
  },                                                                                                  // 113
                                                                                                      // 114
  createAccountForIdentity: function() {                                                              // 115
    // Creates a new account for the currently-logged-in identity.                                    // 116
                                                                                                      // 117
    var user = Meteor.user();                                                                         // 118
    if (!(user && user.profile)) {                                                                    // 119
      throw new Meteor.Error(403, "Must be logged in as an identity in order to create an account.");
    }                                                                                                 // 121
                                                                                                      // 122
    if (Meteor.users.findOne({$or: [{"loginIdentities.id": user._id},                                 // 123
                                    {"nonloginIdentities.id": user._id}]})) {                         // 124
      throw new Meteor.Error(403, "Cannot create an account for an identity that's already " +        // 125
                                  "linked to another account.");                                      // 126
    }                                                                                                 // 127
                                                                                                      // 128
    var newUser = {loginIdentities: [{id: user._id}],                                                 // 129
                   nonloginIdentities: []};                                                           // 130
    if (user.services.dev) {                                                                          // 131
      newUser.signupKey = "devAccounts";                                                              // 132
      if (user.services.dev.isAdmin) {                                                                // 133
        newUser.isAdmin = true;                                                                       // 134
      }                                                                                               // 135
      if (user.services.dev.hasCompletedSignup) {                                                     // 136
        newUser.hasCompletedSignup = true;                                                            // 137
      }                                                                                               // 138
    } else if (user.expires) {                                                                        // 139
      // Demo user.                                                                                   // 140
      newUser.expires = user.expires;                                                                 // 141
    }                                                                                                 // 142
    var options = {};                                                                                 // 143
                                                                                                      // 144
    // This will throw an error if the identity has been added as a login identity to some            // 145
    // other account while we were executing the body of this method.                                 // 146
    return Accounts.insertUserDoc(options, newUser);                                                  // 147
  },                                                                                                  // 148
                                                                                                      // 149
  linkIdentityToAccount: function(token) {                                                            // 150
    // Links the identity of the current user to the account that has `token` as a resume token.      // 151
    // If the account is a demo account, makes the account durable and gives the identity login       // 152
    // access to it.                                                                                  // 153
                                                                                                      // 154
    check(token, String);                                                                             // 155
                                                                                                      // 156
    if (!this.userId) {                                                                               // 157
      throw new Meteor.Error(403, "Cannot link to account if not logged in.");                        // 158
    }                                                                                                 // 159
    var hashed = Accounts._hashLoginToken(token);                                                     // 160
    var accountUser = Meteor.users.findOne({"services.resume.loginTokens.hashedToken": hashed});      // 161
                                                                                                      // 162
    linkIdentityToAccountInternal(this.userId, accountUser._id);                                      // 163
  },                                                                                                  // 164
                                                                                                      // 165
  unlinkIdentity: function (accountUserId, identityId) {                                              // 166
    // Unlinks the identity with ID `identityId` from the account with ID `accountUserId`.            // 167
                                                                                                      // 168
    check(identityId, String);                                                                        // 169
    check(accountUserId, String);                                                                     // 170
                                                                                                      // 171
    if (!this.userId) {                                                                               // 172
      throw new Meteor.Error(403, "Not logged in.");                                                  // 173
    }                                                                                                 // 174
    if (!this.connection.sandstormDb.userHasIdentity(this.userId, identityId)) {                      // 175
      throw new Meteor.Error(403, "Current user does not own identity " + identityId);                // 176
    }                                                                                                 // 177
                                                                                                      // 178
    var identityUser = Meteor.users.findOne({_id: identityId});                                       // 179
    Meteor.users.update({_id: accountUserId},                                                         // 180
                        {$pull: {nonloginIdentities: {id: identityId},                                // 181
                                 loginIdentities: {id: identityId}}});                                // 182
  },                                                                                                  // 183
                                                                                                      // 184
  setIdentityAllowsLogin: function(identityId, allowLogin) {                                          // 185
    // Sets whether the current account allows the identity with ID `identityId` to log in.           // 186
                                                                                                      // 187
    check(identityId, String);                                                                        // 188
    check(allowLogin, Boolean);                                                                       // 189
    if (!this.userId) {                                                                               // 190
      throw new Meteor.Error(403, "Not logged in.");                                                  // 191
    }                                                                                                 // 192
    if (!this.connection.sandstormDb.userHasIdentity(this.userId, identityId)) {                      // 193
      throw new Meteor.Error(403, "Current user does not own identity " + identityId);                // 194
    }                                                                                                 // 195
                                                                                                      // 196
    if (allowLogin) {                                                                                 // 197
      Meteor.users.update({_id: this.userId,                                                          // 198
                           "nonloginIdentities.id": identityId,                                       // 199
                           "loginIdentities.id": {$not: {$eq: identityId}}},                          // 200
                          {$pull: {nonloginIdentities: {id: identityId}},                             // 201
                           $push: {loginIdentities: {id: identityId}}});                              // 202
    } else {                                                                                          // 203
      Meteor.users.update({_id: this.userId,                                                          // 204
                           "loginIdentities.id": identityId,                                          // 205
                           "nonloginIdentities.id": {$not: {$eq: identityId}}},                       // 206
                          {$pull: {loginIdentities: {id: identityId}},                                // 207
                           $push: {nonloginIdentities: {id: identityId}}});                           // 208
    }                                                                                                 // 209
  },                                                                                                  // 210
                                                                                                      // 211
  logoutIdentitiesOfCurrentAccount: function() {                                                      // 212
    // Logs out all identities that are allowed to log in to the current account.                     // 213
    var user = Meteor.user();                                                                         // 214
    if (user && user.loginIdentities) {                                                               // 215
      user.loginIdentities.forEach(function(identity) {                                               // 216
        Meteor.users.update({_id: identity.id}, {$set: {"services.resume.loginTokens": []}});         // 217
      });                                                                                             // 218
    }                                                                                                 // 219
  }                                                                                                   // 220
});                                                                                                   // 221
                                                                                                      // 222
Accounts.linkIdentityToAccount = function (identityId, accountId) {                                   // 223
  // Links the identity to the account. If the account is a demo account, makes it durable and        // 224
  // gives the identity login access to it.                                                           // 225
  check(identityId, String);                                                                          // 226
  check(accountId, String);                                                                           // 227
  linkIdentityToAccountInternal(identityId, accountId);                                               // 228
}                                                                                                     // 229
                                                                                                      // 230
Meteor.publish("accountsOfIdentity", function (identityId) {                                          // 231
  check(identityId, String);                                                                          // 232
  if (!SandstormDb.ensureSubscriberHasIdentity(this, identityId)) return;                             // 233
                                                                                                      // 234
  // We maintain a map from identity IDs to live query handles that track profile changes.            // 235
  var loginIdentities = {};                                                                           // 236
                                                                                                      // 237
  var self = this;                                                                                    // 238
  function addIdentitiesOfAccount(account) {                                                          // 239
    account.loginIdentities.forEach(function(identity) {                                              // 240
      if (!(identity.id in loginIdentities)) {                                                        // 241
        var user = Meteor.users.findOne({_id: identity.id});                                          // 242
        if (user) {                                                                                   // 243
          SandstormDb.fillInProfileDefaults(user);                                                    // 244
          SandstormDb.fillInIntrinsicName(user);                                                      // 245
          var filteredUser = _.pick(user, "_id", "profile");                                          // 246
          filteredUser.loginAccountId = account._id;                                                  // 247
          filteredUser.sourceIdentityId = identityId;                                                 // 248
          self.added("loginIdentitiesOfLinkedAccounts", user._id, filteredUser);                      // 249
        }                                                                                             // 250
        loginIdentities[identity.id] =                                                                // 251
          Meteor.users.find({_id: identity.id}, {fields: {profile: 1}}).observeChanges({              // 252
            changed: function (id, fields) {                                                          // 253
              self.changed("loginIdentitiesOfLinkedAccounts", id, fields);                            // 254
            }                                                                                         // 255
          });                                                                                         // 256
      }                                                                                               // 257
    });                                                                                               // 258
  }                                                                                                   // 259
  var cursor = Meteor.users.find({$or: [{"loginIdentities.id": identityId},                           // 260
                                        {"nonloginIdentities.id": identityId}]});                     // 261
                                                                                                      // 262
  var handle = cursor.observe({                                                                       // 263
    added: function (account) {                                                                       // 264
      addIdentitiesOfAccount(account);                                                                // 265
    },                                                                                                // 266
    changed: function (newAccount, oldAccount) {                                                      // 267
      addIdentitiesOfAccount(newAccount);                                                             // 268
    },                                                                                                // 269
    removed: function (account) {                                                                     // 270
      account.loginIdentities.forEach(function(identity) {                                            // 271
        if (identity.id in loginIdentities) {                                                         // 272
          self.removed("loginIdentitiesOfLinkedAccounts", identity.id);                               // 273
          loginIdentities[identity.id].stop();                                                        // 274
          delete loginIdentities[identity.id];                                                        // 275
        }                                                                                             // 276
      });                                                                                             // 277
    },                                                                                                // 278
  });                                                                                                 // 279
  this.ready();                                                                                       // 280
                                                                                                      // 281
  this.onStop(function() {                                                                            // 282
    handle.stop();                                                                                    // 283
    Object.keys(loginIdentities).forEach(function(identityId) {                                       // 284
      loginIdentities[identityId].stop();                                                             // 285
      delete loginIdentities[identityId];                                                             // 286
    });                                                                                               // 287
  });                                                                                                 // 288
});                                                                                                   // 289
                                                                                                      // 290
////////////////////////////////////////////////////////////////////////////////////////////////////////

}).call(this);


/* Exports */
if (typeof Package === 'undefined') Package = {};
Package['accounts-identity'] = {};

})();

//# sourceMappingURL=accounts-identity.js.map
